@@ -1,6 +1,7 @@
 #![deny(
     clippy::suspicious,
     clippy::style,
+    rustdoc::broken_intra_doc_links,
     missing_debug_implementations,
     missing_copy_implementations
 )]
@@ -22,6 +23,7 @@ use log::{error, LevelFilter};
 use tokio::runtime::Builder as RtBuilder;
 
 mod client;
+mod format;
 mod server;
 
 type Result<T = (), E = Error> = std::result::Result<T, E>;
@@ -45,6 +47,12 @@ lazy_static! {
     static ref SERVER_PATH: String = format!("{}/Daemon", *PATH_PREFIX);
 }
 
+pub(crate) mod metadata {
+    pub const PLAYER_BUS: &str = "empress:playerBus";
+    pub const PLAYER_IDENTITY: &str = "empress:playerIdentity";
+    pub const POSITION: &str = "empress:position";
+}
+
 #[derive(Debug, Clone, Copy, strum::Display)]
 enum MethodId {
     /// List the players currently tracked by the daemon
@@ -109,7 +117,16 @@ enum ClientCommand {
     /// Skip one track forwards
     Next(PlayerOpts),
     /// Print information about the current track
-    NowPlaying(PlayerOpts),
+    #[clap(long_about = include_str!("../etc/now_playing_long.txt"))]
+    NowPlaying {
+        #[clap(flatten)]
+        player: PlayerOpts,
+
+        /// Instead of outputting JSON, output a plaintext string with the given
+        /// format.  See the full help for this command for a syntax reference.
+        #[clap(short, long)]
+        format: Option<String>,
+    },
     /// Skip one track backwards
     Previous(PlayerOpts),
     /// Pause a currently-playing player
@@ -245,7 +262,7 @@ impl ::std::str::FromStr for Offset {
     }
 }
 
-fn log_cfg(v: usize) -> env_logger::Builder {
+fn log_init(v: usize, f: impl FnOnce(&mut env_logger::Builder)) -> Result {
     const VERBOSITY: [LevelFilter; 3] = [LevelFilter::Info, LevelFilter::Debug, LevelFilter::Trace];
     #[cfg(debug_assertions)]
     const DEFAULT_V: usize = 1;
@@ -254,20 +271,23 @@ fn log_cfg(v: usize) -> env_logger::Builder {
 
     let mut b = env_logger::builder();
 
-    b.filter_level(VERBOSITY[(DEFAULT_V + v).min(VERBOSITY.len() - 1)])
-        .parse_default_env();
+    b.filter_level(VERBOSITY[(DEFAULT_V + v).min(VERBOSITY.len() - 1)]);
 
-    b
+    f(&mut b);
+
+    b.parse_default_env().try_init().map_err(Into::into)
 }
 
 fn main() {
     let result = run();
 
-    log_cfg(0).try_init().ok();
-
     match result {
         Ok(()) => (),
-        Err(e) => error!("{:?}", e),
+        Err(e) => {
+            log_init(0, |_| ()).ok();
+
+            error!("{:?}", e);
+        },
     }
 }
 
@@ -285,18 +305,17 @@ fn run() -> Result {
             no_quiet,
             verbose,
         } => {
-            let mut log_cfg = log_cfg(verbose);
-
-            if !(no_quiet || verbose != 0 || atty::is(atty::Stream::Stderr)) || quiet {
-                log_cfg.filter_level(LevelFilter::Warn);
-            }
-
-            log_cfg.init();
+            log_init(verbose, |b| {
+                if !(no_quiet || verbose != 0 || atty::is(atty::Stream::Stderr)) || quiet {
+                    b.filter_level(LevelFilter::Warn);
+                }
+            })
+            .unwrap();
 
             rt.block_on(server::run())
         },
         Opts::Client(id) => {
-            log_cfg(0).init();
+            log_init(0, |_| ()).unwrap();
 
             rt.block_on(client::run(id))
         },
