@@ -15,7 +15,7 @@ use dbus::{
     strings::{BusName, Path},
 };
 use futures::prelude::*;
-use log::{debug, warn};
+use log::{debug, trace, warn};
 use tokio::{
     select,
     sync::{mpsc, RwLock},
@@ -95,7 +95,11 @@ impl Server {
     }
 
     async fn scan(&self, force: bool) -> Result {
-        debug!("Running {} scan...", if force { "full" } else { "quick" });
+        if force {
+            debug!("Running full scan...");
+        } else {
+            trace!("Running quick scan...");
+        }
 
         let proxy = Proxy::new(
             "org.freedesktop.DBus",
@@ -122,7 +126,7 @@ impl Server {
         )
         .await;
 
-        debug!("{} scan completed", if force { "Full" } else { "Quick" });
+        trace!("Scan completed.");
 
         Ok(())
     }
@@ -138,8 +142,7 @@ impl Server {
         let mut players = self.players.write().await;
         let mut patch = Err(None);
 
-        // TODO: use drain_filter or something less stupid than this
-        for player in players.iter() {
+        for player in players.iter_active() {
             match f(player.clone()).await {
                 Ok(Some(next)) => {
                     patch = Ok(Some(next));
@@ -171,7 +174,7 @@ impl Server {
         let players = self.players.read().await;
         let mut default = Err(None);
 
-        for player in players.iter() {
+        for player in players.iter_active() {
             match f(player.clone()).await {
                 ok @ Ok(Some(_)) => return ok,
                 Ok(None) => default = Ok(None),
@@ -220,7 +223,7 @@ impl Server {
                     .players
                     .read()
                     .await
-                    .iter()
+                    .iter_all()
                     .filter_map(|p| {
                         p.bus
                             .strip_prefix(&**mpris::BUS_NAME)
@@ -256,16 +259,39 @@ impl Server {
                                         .map_or(false, |p| p == *mpris::track_list::NO_TRACK))
                             });
 
-                    Ok(if has_track {
-                        Some((
-                            meta.into_iter()
-                                .map(|(k, Variant(v))| (k, Variant(v.box_clone())))
-                                .collect(),
-                            p.status,
-                        ))
+                    let bus = p
+                        .bus
+                        .strip_prefix(&**mpris::BUS_NAME)
+                        .and_then(|s| s.strip_prefix('.'))
+                        .map_or_else(String::new, Into::into);
+                    let ident = p.identity(&*self.conn).await?;
+
+                    // Properties that should go into the map regardless of if we have a track
+                    let extra_props: [(String, Variant<Box<dyn RefArg>>); 2] = [
+                        (crate::metadata::PLAYER_BUS.into(), Variant(Box::new(bus))),
+                        (
+                            crate::metadata::PLAYER_IDENTITY.into(),
+                            Variant(Box::new(ident)),
+                        ),
+                    ];
+
+                    Ok(Some(if has_track {
+                        let mut meta: HashMap<_, _> = meta
+                            .into_iter()
+                            .map(|(k, Variant(v))| (k, Variant(v.box_clone())))
+                            .chain(extra_props)
+                            .collect();
+
+                        let pos = p.position(&*self.conn).await.ok();
+
+                        if let Some(pos) = pos {
+                            meta.insert(crate::metadata::POSITION.into(), Variant(Box::new(pos)));
+                        }
+
+                        (meta, p.status)
                     } else {
-                        None
-                    })
+                        (extra_props.into_iter().collect(), p.status)
+                    }))
                 })
                 .map_ok(|ok| {
                     let (map, status) =
@@ -431,7 +457,7 @@ impl Server {
                     let mut put = vec![];
                     let conn = &*self.conn;
 
-                    for player in self.players.read().await.iter() {
+                    for player in self.players.read().await.iter_all() {
                         if player.playback_status(conn).await? == PlaybackStatus::Playing
                             && player.can_pause(conn).await?
                         {
